@@ -1,3 +1,4 @@
+use std::env;
 ///! Create ld memory sections programmaticaly
 ///
 /// This crate can be used in build.rs scripts to replace static memory.x files
@@ -195,6 +196,23 @@ impl MemorySection {
             }
         }
 
+        // If being called by cargo, assume we're running from build.rs.
+        // Thus, print "cargo:rerun..." lines.
+        // Here we're assuming that if both CARGO and OUT_DIR is set, we're in
+        // build.rs.
+        if env::var("CARGO").is_ok() && env::var("OUT_DIR").is_ok() {
+            for var in [
+                offset_env,
+                num_slots_env,
+                slot_env,
+                slot_offset_env,
+                pagesize_env,
+            ]
+            .iter()
+            {
+                println!("cargo:rerun-if-env-changed={}", var);
+            }
+        }
         res
     }
 
@@ -250,7 +268,7 @@ mod tests {
         let section = MemorySection::new("SectionName", 0, 0xFFFF);
         assert_eq!(
             section.to_string(),
-            "    SectionName : ORIGIN = 0x00000000, LENGTH = 65535\n"
+            "    SectionName : ORIGIN = 0x0, LENGTH = 0xFFFF\n"
         );
     }
 
@@ -259,7 +277,7 @@ mod tests {
         let section = MemorySection::new("SectionName", 0, 0x10000).offset(0x1000);
         assert_eq!(
             section.to_string(),
-            "    SectionName : ORIGIN = 0x00001000, LENGTH = 61440\n"
+            "    SectionName : ORIGIN = 0x1000, LENGTH = 0xF000\n"
         );
     }
 
@@ -268,7 +286,7 @@ mod tests {
         let section = MemorySection::new("SectionName", 0, 0x10000).attrs("r!w!x");
         assert_eq!(
             section.to_string(),
-            "    SectionName (r!w!x): ORIGIN = 0x00000000, LENGTH = 65536\n"
+            "    SectionName (r!w!x): ORIGIN = 0x0, LENGTH = 0x10000\n"
         );
     }
 
@@ -284,7 +302,7 @@ mod tests {
             memory.to_string(),
             concat!(
                 "MEMORY\n{\n",
-                "    SectionName (rw!x): ORIGIN = 0x00001000, LENGTH = 61440\n",
+                "    SectionName (rw!x): ORIGIN = 0x1000, LENGTH = 0xF000\n",
                 "}\n"
             )
         );
